package com.jiuqi.syntax;

import java.util.GregorianCalendar;
import java.math.BigDecimal;
import java.util.Collection;
import java.util.ArrayList;
import java.util.Stack;

public final class NiporlanParser
{
    public static int NEW_COUNT;
    private static final int ASSIGN_CELL = 0;
    private Stack stack;
    private SyntaxParser syntaxParser;
    private SyntaxEvent syntaxEvent;
    private CommonData cellValue;
    private boolean useDataLocator;
    private ArrayList usedNiporlanItems;
    private ArrayList unUsedNiporlanItems;
    
    static {
        NiporlanParser.NEW_COUNT = 0;
    }
    
    public NiporlanParser(final SyntaxParser syntaxParser) {
        this.stack = new Stack();
        this.syntaxEvent = new SyntaxEvent(this);
        this.cellValue = new CommonData();
        this.useDataLocator = false;
        this.usedNiporlanItems = new ArrayList();
        this.unUsedNiporlanItems = new ArrayList();
        ++NiporlanParser.NEW_COUNT;
        this.syntaxParser = syntaxParser;
    }
    
    public boolean getUseDataLocator() {
        return this.useDataLocator;
    }
    
    public void setUseDataLocator(final boolean value) {
        this.useDataLocator = value;
    }
    
    private NiporlanItem newNiporlanItem() {
        NiporlanItem niporlanItem;
        if (this.unUsedNiporlanItems.size() > 0) {
            niporlanItem = this.unUsedNiporlanItems.get(this.unUsedNiporlanItems.size() - 1);
            this.unUsedNiporlanItems.remove(niporlanItem);
            this.usedNiporlanItems.add(niporlanItem);
        }
        else {
            niporlanItem = new NiporlanItem();
            this.usedNiporlanItems.add(niporlanItem);
        }
        return niporlanItem;
    }
    
    private void disposeNiporlanItem(final NiporlanItem niporlanItem) {
        final int index = this.usedNiporlanItems.indexOf(niporlanItem);
        if (index >= 0) {
            this.usedNiporlanItems.remove(index);
            this.unUsedNiporlanItems.add(niporlanItem);
        }
    }
    
    private void disposeAllNiporlanItem() {
        this.unUsedNiporlanItems.addAll(this.usedNiporlanItems);
        this.usedNiporlanItems.clear();
    }
    
    public boolean judge(final Niporlan niporlan) {
        if (niporlan.isEmpty()) {
            return true;
        }
        if (niporlan.ifExist()) {
            this.kernel(niporlan.ifItems, false);
            if (((NiporlanItem)this.stack.get(0)).getCommonData().cdBool) {
                this.kernel(niporlan.thenItems, false);
            }
            else if (niporlan.elseItems != null) {
                this.kernel(niporlan.elseItems, false);
            }
            else {
                this.stack.clear();
            }
            return this.stack.size() == 0 || ((NiporlanItem)this.stack.get(0)).getCommonData().cdBool;
        }
        this.kernel(niporlan.thenItems, false);
        return this.stack.size() == 0 || ((NiporlanItem)this.stack.get(0)).getCommonData().cdBool;
    }
    
    public void evaluate(final Niporlan niporlan) {
        if (niporlan.isEmpty()) {
            return;
        }
        if (niporlan.ifExist()) {
            this.kernel(niporlan.ifItems, false);
            if (((NiporlanItem)this.stack.get(0)).getCommonData().cdBool) {
                this.kernel(niporlan.thenItems, true);
            }
            else if (niporlan.elseItems != null) {
                this.kernel(niporlan.elseItems, true);
            }
            else {
                this.stack.clear();
            }
        }
        else {
            this.kernel(niporlan.thenItems, true);
        }
    }
    
    public void attract(final Niporlan niporlan, final CommonData resultData) {
        if (niporlan.isEmpty()) {
            resultData.type = 0;
            resultData.cdReal = 0.0;
            return;
        }
        if (niporlan.ifExist()) {
            this.kernel(niporlan.ifItems, false);
            if (((NiporlanItem)this.stack.get(0)).getCommonData().cdBool) {
                this.kernel(niporlan.thenItems, false);
            }
            else if (niporlan.elseItems != null) {
                this.kernel(niporlan.elseItems, false);
            }
            else {
                this.stack.clear();
            }
            if (this.stack.size() == 0) {
                resultData.type = 0;
            }
            else {
                resultData.assign(((NiporlanItem)this.stack.get(0)).getCommonData());
            }
        }
        else {
            if (niporlan.thenItems.size() == 1) {
                final NiporlanItem ni = niporlan.thenItems.get(0);
                if (ni.getType() == 3) {
                    this.syntaxEvent.dataCell = ni.getDataCell();
                    this.syntaxEvent.value = this.cellValue;
                    if (this.useDataLocator) {
                        this.getCellValue(this.syntaxEvent, ni);
                    }
                    else {
                        this.getCellValue(this.syntaxEvent);
                    }
                    resultData.assign(this.syntaxEvent.value);
                    if (resultData.type == 1) {
                        resultData.type = 2;
                        resultData.cdReal = resultData.cdInt;
                    }
                    if (resultData.type == 0) {
                        this.error("Niporlan公式有错误(空数据类型)！");
                    }
                    return;
                }
            }
            this.kernel(niporlan.thenItems, false);
            if (this.stack.size() == 0) {
                resultData.type = 0;
            }
            else {
                resultData.assign(((NiporlanItem)this.stack.get(0)).getCommonData());
            }
        }
    }
    
    private void bool2Real(final CommonData data) {
        if (data.type == 5) {
            data.type = 2;
            if (data.cdBool) {
                data.cdReal = -1.0;
            }
            else {
                data.cdReal = 0.0;
            }
        }
    }
    
    private void real2Decimal(final CommonData data) {
        if (data.type == 2) {
            data.type = 10;
            data.cdDecimal = new BigDecimal(data.cdReal);
        }
    }
    
    private void kernel(final ArrayList niItems, final boolean isAssignValue) {
        int assignKind = 0;
        this.stack.clear();
        this.disposeAllNiporlanItem();
        for (int i = 0, n = niItems.size(); i < n; ++i) {
            final NiporlanItem item = niItems.get(i);
            switch (item.getType()) {
                case 0: {
                    switch (item.getOp()) {
                        case 1: {
                            final NiporlanItem rightItem = this.stack.pop();
                            final NiporlanItem leftItem = this.stack.peek();
                            final CommonData leftItemCommonData = leftItem.getCommonData();
                            final CommonData rightItemCommonData = rightItem.getCommonData();
                            this.bool2Real(rightItemCommonData);
                            this.bool2Real(leftItemCommonData);
                            if (rightItemCommonData.type == 10) {
                                this.real2Decimal(leftItemCommonData);
                            }
                            switch (leftItemCommonData.type) {
                                case 2: {
                                    leftItemCommonData.cdReal += rightItemCommonData.cdReal;
                                    break;
                                }
                                case 3: {
                                    leftItemCommonData.cdString = String.valueOf(leftItemCommonData.cdString) + rightItemCommonData.cdString;
                                    break;
                                }
                                case 10: {
                                    this.real2Decimal(rightItemCommonData);
                                    if (leftItemCommonData.isNull) {
                                        leftItemCommonData.cdDecimal = rightItemCommonData.cdDecimal;
                                        break;
                                    }
                                    if (!rightItemCommonData.isNull) {
                                        leftItemCommonData.cdDecimal = leftItemCommonData.cdDecimal.add(rightItemCommonData.cdDecimal);
                                        break;
                                    }
                                    break;
                                }
                            }
                            leftItemCommonData.isNull = (leftItemCommonData.isNull && rightItemCommonData.isNull);
                            leftItemCommonData.initNullValue();
                            this.disposeNiporlanItem(rightItem);
                            continue;
                        }
                        case 2: {
                            final NiporlanItem rightItem = this.stack.pop();
                            final NiporlanItem leftItem = this.stack.peek();
                            final CommonData leftItemCommonData = leftItem.getCommonData();
                            final CommonData rightItemCommonData = rightItem.getCommonData();
                            this.bool2Real(rightItemCommonData);
                            this.bool2Real(leftItemCommonData);
                            if (rightItemCommonData.type == 10) {
                                this.real2Decimal(leftItemCommonData);
                            }
                            switch (leftItemCommonData.type) {
                                case 2: {
                                    leftItemCommonData.cdReal -= rightItemCommonData.cdReal;
                                    if (Math.abs(leftItemCommonData.cdReal) < 1.0E-5) {
                                        leftItemCommonData.cdReal = 0.0;
                                        break;
                                    }
                                    break;
                                }
                                case 6: {
                                    leftItemCommonData.type = 2;
                                    leftItemCommonData.cdReal = SyntaxParser.minusDate(leftItemCommonData.cdDate, rightItemCommonData.cdDate);
                                    if (Math.abs(leftItemCommonData.cdReal) < 1.0E-5) {
                                        leftItemCommonData.cdReal = 0.0;
                                        break;
                                    }
                                    break;
                                }
                                case 10: {
                                    this.real2Decimal(rightItemCommonData);
                                    if (leftItemCommonData.isNull) {
                                        leftItemCommonData.cdDecimal = rightItemCommonData.cdDecimal.negate();
                                        break;
                                    }
                                    if (!rightItemCommonData.isNull) {
                                        leftItemCommonData.cdDecimal = leftItemCommonData.cdDecimal.subtract(rightItemCommonData.cdDecimal);
                                        break;
                                    }
                                    break;
                                }
                            }
                            leftItemCommonData.isNull = (leftItemCommonData.isNull && rightItemCommonData.isNull);
                            leftItemCommonData.initNullValue();
                            this.disposeNiporlanItem(rightItem);
                            continue;
                        }
                        case 3: {
                            final NiporlanItem leftItem = this.stack.peek();
                            final CommonData leftItemCommonData = leftItem.getCommonData();
                            this.bool2Real(leftItemCommonData);
                            if (leftItemCommonData.type == 10) {
                                if (!leftItemCommonData.isNull) {
                                    leftItemCommonData.cdDecimal = leftItemCommonData.cdDecimal.negate();
                                }
                            }
                            else {
                                leftItem.getCommonData().cdReal = -leftItem.getCommonData().cdReal;
                            }
                            leftItem.getCommonData().initNullValue();
                            continue;
                        }
                        case 4: {
                            final NiporlanItem rightItem = this.stack.pop();
                            final NiporlanItem leftItem = this.stack.peek();
                            final CommonData leftItemCommonData = leftItem.getCommonData();
                            final CommonData rightItemCommonData = rightItem.getCommonData();
                            this.bool2Real(rightItemCommonData);
                            this.bool2Real(leftItemCommonData);
                            if (rightItemCommonData.type == 10) {
                                this.real2Decimal(leftItemCommonData);
                            }
                            if (leftItemCommonData.type == 10) {
                                if (!leftItemCommonData.isNull && !rightItemCommonData.isNull) {
                                    this.real2Decimal(rightItemCommonData);
                                    leftItemCommonData.cdDecimal = leftItemCommonData.cdDecimal.multiply(rightItemCommonData.cdDecimal);
                                }
                            }
                            else {
                                leftItemCommonData.cdReal *= rightItemCommonData.cdReal;
                            }
                            leftItemCommonData.isNull = (leftItemCommonData.isNull || rightItemCommonData.isNull);
                            leftItemCommonData.initNullValue();
                            this.disposeNiporlanItem(rightItem);
                            continue;
                        }
                        case 5: {
                            final NiporlanItem rightItem = this.stack.pop();
                            final NiporlanItem leftItem = this.stack.peek();
                            final CommonData leftItemCommonData = leftItem.getCommonData();
                            final CommonData rightItemCommonData = rightItem.getCommonData();
                            this.bool2Real(rightItemCommonData);
                            this.bool2Real(leftItemCommonData);
                            if (rightItemCommonData.type == 10) {
                                this.real2Decimal(leftItemCommonData);
                            }
                            if (leftItemCommonData.type == 10) {
                                if (!leftItemCommonData.isNull && !rightItemCommonData.isNull) {
                                    this.real2Decimal(rightItemCommonData);
                                    if (rightItemCommonData.cdDecimal.abs().compareTo(new BigDecimal(1.0E-5)) < 0) {
                                        leftItemCommonData.cdDecimal = new BigDecimal(0);
                                    }
                                    else {
                                        leftItemCommonData.cdDecimal = leftItemCommonData.cdDecimal.divide(rightItemCommonData.cdDecimal, 4);
                                    }
                                }
                            }
                            else if (Math.abs(rightItemCommonData.cdReal) < 1.0E-5) {
                                leftItemCommonData.cdReal = 0.0;
                            }
                            else {
                                leftItemCommonData.cdReal /= rightItemCommonData.cdReal;
                            }
                            leftItemCommonData.isNull = (leftItemCommonData.isNull || rightItemCommonData.isNull);
                            leftItemCommonData.initNullValue();
                            this.disposeNiporlanItem(rightItem);
                            continue;
                        }
                        case 7: {
                            final NiporlanItem rightItem = this.stack.pop();
                            final NiporlanItem leftItem = this.stack.peek();
                            final CommonData leftItemCommonData = leftItem.getCommonData();
                            final CommonData rightItemCommonData = rightItem.getCommonData();
                            if (rightItemCommonData.type == 10) {
                                this.real2Decimal(leftItemCommonData);
                            }
                            if (leftItemCommonData.type == 10) {
                                this.real2Decimal(rightItemCommonData);
                            }
                            leftItemCommonData.cdBool = leftItemCommonData.equals(rightItemCommonData, this.syntaxParser.getBalanceDifferenceValue());
                            leftItemCommonData.type = 5;
                            leftItemCommonData.isNull = false;
                            this.disposeNiporlanItem(rightItem);
                            continue;
                        }
                        case 8: {
                            continue;
                        }
                        case 9: {
                            final NiporlanItem rightItem = this.stack.pop();
                            final NiporlanItem leftItem = this.stack.peek();
                            final CommonData leftItemCommonData = leftItem.getCommonData();
                            final CommonData rightItemCommonData = rightItem.getCommonData();
                            if (rightItemCommonData.type == 10) {
                                this.real2Decimal(leftItemCommonData);
                            }
                            if (leftItemCommonData.type == 10) {
                                this.real2Decimal(rightItemCommonData);
                            }
                            leftItem.getCommonData().cdBool = !leftItem.getCommonData().equals(rightItem.getCommonData(), this.syntaxParser.getBalanceDifferenceValue());
                            leftItem.getCommonData().type = 5;
                            leftItem.getCommonData().isNull = false;
                            this.disposeNiporlanItem(rightItem);
                            continue;
                        }
                        case 10: {
                            final NiporlanItem rightItem = this.stack.pop();
                            final NiporlanItem leftItem = this.stack.peek();
                            final CommonData leftItemCommonData = leftItem.getCommonData();
                            final CommonData rightItemCommonData = rightItem.getCommonData();
                            if (rightItemCommonData.type == 10) {
                                this.real2Decimal(leftItemCommonData);
                            }
                            if (leftItemCommonData.type == 10) {
                                this.real2Decimal(rightItemCommonData);
                            }
                            leftItem.getCommonData().cdBool = (leftItem.getCommonData().compareTo(rightItem.getCommonData(), 1.0E-5) > 0);
                            leftItem.getCommonData().type = 5;
                            leftItem.getCommonData().isNull = false;
                            this.disposeNiporlanItem(rightItem);
                            continue;
                        }
                        case 11: {
                            final NiporlanItem rightItem = this.stack.pop();
                            final NiporlanItem leftItem = this.stack.peek();
                            final CommonData leftItemCommonData = leftItem.getCommonData();
                            final CommonData rightItemCommonData = rightItem.getCommonData();
                            if (rightItemCommonData.type == 10) {
                                this.real2Decimal(leftItemCommonData);
                            }
                            if (leftItemCommonData.type == 10) {
                                this.real2Decimal(rightItemCommonData);
                            }
                            leftItem.getCommonData().cdBool = (leftItem.getCommonData().compareTo(rightItem.getCommonData(), 1.0E-5) < 0);
                            leftItem.getCommonData().type = 5;
                            leftItem.getCommonData().isNull = false;
                            this.disposeNiporlanItem(rightItem);
                            continue;
                        }
                        case 12: {
                            final NiporlanItem rightItem = this.stack.pop();
                            final NiporlanItem leftItem = this.stack.peek();
                            final CommonData leftItemCommonData = leftItem.getCommonData();
                            final CommonData rightItemCommonData = rightItem.getCommonData();
                            if (rightItemCommonData.type == 10) {
                                this.real2Decimal(leftItemCommonData);
                            }
                            if (leftItemCommonData.type == 10) {
                                this.real2Decimal(rightItemCommonData);
                            }
                            leftItem.getCommonData().cdBool = (leftItem.getCommonData().compareTo(rightItem.getCommonData(), this.syntaxParser.getBalanceDifferenceValue()) >= 0);
                            leftItem.getCommonData().type = 5;
                            leftItem.getCommonData().isNull = false;
                            this.disposeNiporlanItem(rightItem);
                            continue;
                        }
                        case 13: {
                            final NiporlanItem rightItem = this.stack.pop();
                            final NiporlanItem leftItem = this.stack.peek();
                            final CommonData leftItemCommonData = leftItem.getCommonData();
                            final CommonData rightItemCommonData = rightItem.getCommonData();
                            if (rightItemCommonData.type == 10) {
                                this.real2Decimal(leftItemCommonData);
                            }
                            if (leftItemCommonData.type == 10) {
                                this.real2Decimal(rightItemCommonData);
                            }
                            leftItem.getCommonData().cdBool = (leftItem.getCommonData().compareTo(rightItem.getCommonData(), this.syntaxParser.getBalanceDifferenceValue()) <= 0);
                            leftItem.getCommonData().type = 5;
                            leftItem.getCommonData().isNull = false;
                            this.disposeNiporlanItem(rightItem);
                            continue;
                        }
                        case 14: {
                            final NiporlanItem rightItem = this.stack.pop();
                            final NiporlanItem leftItem = this.stack.peek();
                            leftItem.getCommonData().cdBool = (leftItem.getCommonData().cdBool && rightItem.getCommonData().cdBool);
                            leftItem.getCommonData().isNull = (leftItem.getCommonData().isNull && rightItem.getCommonData().isNull);
                            leftItem.getCommonData().initNullValue();
                            this.disposeNiporlanItem(rightItem);
                            continue;
                        }
                        case 15: {
                            final NiporlanItem rightItem = this.stack.pop();
                            final NiporlanItem leftItem = this.stack.peek();
                            leftItem.getCommonData().cdBool = (leftItem.getCommonData().cdBool || rightItem.getCommonData().cdBool);
                            leftItem.getCommonData().isNull = (leftItem.getCommonData().isNull && rightItem.getCommonData().isNull);
                            leftItem.getCommonData().initNullValue();
                            this.disposeNiporlanItem(rightItem);
                            continue;
                        }
                        case 16: {
                            final NiporlanItem leftItem = this.stack.peek();
                            leftItem.getCommonData().cdBool = !leftItem.getCommonData().cdBool;
                            leftItem.getCommonData().initNullValue();
                            continue;
                        }
                        case 17: {
                            final NiporlanItem rightItem = this.stack.pop();
                            final NiporlanItem leftItem = this.stack.pop();
                            if (assignKind == 0) {
                                this.syntaxEvent.dataCell = leftItem.getDataCell();
                                this.syntaxEvent.value = rightItem.getCommonData();
                                if (this.useDataLocator) {
                                    this.setCellValue(this.syntaxEvent, leftItem.getDataLocator());
                                }
                                else {
                                    this.setCellValue(this.syntaxEvent);
                                }
                            }
                            this.disposeNiporlanItem(rightItem);
                            this.disposeNiporlanItem(leftItem);
                            continue;
                        }
                        default: {
                            this.error();
                            continue;
                        }
                    }
                    break;
                }
                case 1: {
                    if (item.getFuncName().equals("SUM")) {
                        final NiporlanItem leftItem = this.stack.peek();
                        this.syntaxEvent.dataCell = leftItem.getDataCell();
                        this.syntaxEvent.value = this.cellValue;
                        this.getStatValue(this.syntaxEvent);
                        leftItem.setType(2);
                        leftItem.getCommonData().assign(this.syntaxEvent.value);
                        if (leftItem.getCommonData().type == 1) {
                            leftItem.getCommonData().type = 2;
                            leftItem.getCommonData().cdReal = leftItem.getCommonData().cdInt;
                        }
                        leftItem.getCommonData().initNullValue();
                        break;
                    }
                    if (item.getFuncName().equals("L$")) {
                        final NiporlanItem rightItem = this.stack.pop();
                        final NiporlanItem leftItem = this.stack.peek();
                        leftItem.getCommonData().cdString = SyntaxParser.asciiCopy(leftItem.getCommonData().cdString, 1, (int)rightItem.getCommonData().cdReal);
                        leftItem.getCommonData().initNullValue();
                        this.disposeNiporlanItem(rightItem);
                        break;
                    }
                    if (item.getFuncName().equals("R$")) {
                        final NiporlanItem rightItem = this.stack.pop();
                        final NiporlanItem leftItem = this.stack.peek();
                        if (SyntaxParser.asciiLength(leftItem.getCommonData().cdString) + 1 - (int)rightItem.getCommonData().cdReal >= 1) {
                            leftItem.getCommonData().cdString = SyntaxParser.asciiCopy(leftItem.getCommonData().cdString, SyntaxParser.asciiLength(leftItem.getCommonData().cdString) + 1 - (int)rightItem.getCommonData().cdReal, (int)rightItem.getCommonData().cdReal);
                        }
                        else {
                            leftItem.getCommonData().cdString = leftItem.getCommonData().cdString;
                        }
                        leftItem.getCommonData().initNullValue();
                        this.disposeNiporlanItem(rightItem);
                        break;
                    }
                    if (item.getFuncName().equals("M$")) {
                        final NiporlanItem threeItem = this.stack.pop();
                        final NiporlanItem rightItem = this.stack.pop();
                        final NiporlanItem leftItem = this.stack.peek();
                        leftItem.getCommonData().cdString = SyntaxParser.asciiCopy(leftItem.getCommonData().cdString, (int)rightItem.getCommonData().cdReal, (int)threeItem.getCommonData().cdReal);
                        leftItem.getCommonData().initNullValue();
                        this.disposeNiporlanItem(rightItem);
                        this.disposeNiporlanItem(threeItem);
                        break;
                    }
                    if (item.getFuncName().equals("POS")) {
                        final NiporlanItem rightItem = this.stack.pop();
                        final NiporlanItem leftItem = this.stack.peek();
                        leftItem.getCommonData().cdReal = SyntaxParser.asciiPos(leftItem.getCommonData().cdString, rightItem.getCommonData().cdString);
                        leftItem.getCommonData().type = 2;
                        leftItem.getCommonData().isNull = false;
                        this.disposeNiporlanItem(rightItem);
                        break;
                    }
                    if (item.getFuncName().equals("ABS")) {
                        final NiporlanItem leftItem = this.stack.peek();
                        final CommonData leftItemCommonData = leftItem.getCommonData();
                        if (leftItemCommonData.type == 10) {
                            if (!leftItemCommonData.isNull) {
                                leftItemCommonData.cdDecimal = leftItemCommonData.cdDecimal.abs();
                            }
                        }
                        else {
                            leftItemCommonData.cdReal = Math.abs(leftItem.getCommonData().cdReal);
                        }
                        leftItemCommonData.initNullValue();
                        break;
                    }
                    if (item.getFuncName().equals("INT")) {
                        final NiporlanItem leftItem = this.stack.peek();
                        final CommonData leftItemCommonData = leftItem.getCommonData();
                        if (leftItemCommonData.type == 10) {
                            if (!leftItemCommonData.isNull) {
                                leftItemCommonData.cdReal = leftItemCommonData.cdDecimal.intValue();
                                leftItemCommonData.type = 2;
                                leftItemCommonData.cdDecimal = null;
                            }
                        }
                        else {
                            leftItemCommonData.cdReal = (double)SyntaxParser.trunc(leftItem.getCommonData().cdReal);
                        }
                        leftItemCommonData.initNullValue();
                        break;
                    }
                    if (item.getFuncName().equals("ROUND")) {
                        final NiporlanItem rightItem = this.stack.pop();
                        final NiporlanItem leftItem = this.stack.peek();
                        final CommonData leftItemCommonData = leftItem.getCommonData();
                        if (leftItemCommonData.type == 10) {
                            if (!leftItemCommonData.isNull) {
                                leftItemCommonData.cdDecimal = leftItemCommonData.cdDecimal.setScale((int)rightItem.getCommonData().cdReal, 4);
                            }
                        }
                        else {
                            leftItem.getCommonData().cdReal = SyntaxParser.roundFloat(leftItem.getCommonData().cdReal, (int)rightItem.getCommonData().cdReal);
                        }
                        leftItem.getCommonData().initNullValue();
                        this.disposeNiporlanItem(rightItem);
                        break;
                    }
                    if (item.getFuncName().equals("MOD")) {
                        final NiporlanItem rightItem = this.stack.pop();
                        final NiporlanItem leftItem = this.stack.peek();
                        final CommonData leftItemCommonData = leftItem.getCommonData();
                        if (leftItemCommonData.type == 10) {
                            if (!leftItemCommonData.isNull) {
                                leftItemCommonData.cdReal = SyntaxParser.floatMod(leftItemCommonData.cdDecimal.doubleValue(), rightItem.getCommonData().cdReal);
                                leftItemCommonData.type = 2;
                            }
                        }
                        else {
                            leftItem.getCommonData().cdReal = SyntaxParser.floatMod(leftItem.getCommonData().cdReal, rightItem.getCommonData().cdReal);
                        }
                        leftItem.getCommonData().isNull = (leftItem.getCommonData().isNull || rightItem.getCommonData().isNull);
                        leftItem.getCommonData().initNullValue();
                        this.disposeNiporlanItem(rightItem);
                        break;
                    }
                    if (item.getFuncName().equals("POWER")) {
                        final NiporlanItem rightItem = this.stack.pop();
                        final NiporlanItem leftItem = this.stack.peek();
                        leftItem.getCommonData().cdReal = Math.pow(leftItem.getCommonData().cdReal, rightItem.getCommonData().cdReal);
                        leftItem.getCommonData().isNull = (leftItem.getCommonData().isNull || rightItem.getCommonData().isNull);
                        leftItem.getCommonData().initNullValue();
                        this.disposeNiporlanItem(rightItem);
                        break;
                    }
                    if (item.getFuncName().equals("TAN")) {
                        final NiporlanItem leftItem = this.stack.peek();
                        leftItem.getCommonData().cdReal = Math.tan(leftItem.getCommonData().cdReal);
                        leftItem.getCommonData().initNullValue();
                        break;
                    }
                    if (item.getFuncName().equals("ATAN")) {
                        final NiporlanItem leftItem = this.stack.peek();
                        leftItem.getCommonData().cdReal = Math.atan(leftItem.getCommonData().cdReal);
                        leftItem.getCommonData().initNullValue();
                        break;
                    }
                    if (item.getFuncName().equals("LN")) {
                        final NiporlanItem leftItem = this.stack.peek();
                        leftItem.getCommonData().cdReal = Math.log(leftItem.getCommonData().cdReal);
                        leftItem.getCommonData().initNullValue();
                        break;
                    }
                    if (item.getFuncName().equals("LOG10")) {
                        final NiporlanItem leftItem = this.stack.peek();
                        leftItem.getCommonData().cdReal = Math.log(leftItem.getCommonData().cdReal) / Math.log(10.0);
                        leftItem.getCommonData().initNullValue();
                        break;
                    }
                    if (item.getFuncName().equals("LOG")) {
                        final NiporlanItem rightItem = this.stack.pop();
                        final NiporlanItem leftItem = this.stack.peek();
                        leftItem.getCommonData().cdReal = Math.log(leftItem.getCommonData().cdReal) / Math.log(rightItem.getCommonData().cdReal);
                        leftItem.getCommonData().isNull = (leftItem.getCommonData().isNull || rightItem.getCommonData().isNull);
                        leftItem.getCommonData().initNullValue();
                        this.disposeNiporlanItem(rightItem);
                        break;
                    }
                    if (item.getFuncName().equals("VAL")) {
                        final NiporlanItem leftItem = this.stack.peek();
                        leftItem.getCommonData().cdReal = SyntaxParser.safeStrToFloat(leftItem.getCommonData().cdString);
                        leftItem.getCommonData().type = 2;
                        leftItem.getCommonData().initNullValue();
                        break;
                    }
                    if (item.getFuncName().equals("STR")) {
                        final NiporlanItem threeItem = this.stack.pop();
                        final NiporlanItem rightItem = this.stack.pop();
                        final NiporlanItem leftItem = this.stack.peek();
                        if (leftItem.getCommonData().type == 1 || leftItem.getCommonData().type == 2) {
                            leftItem.getCommonData().cdString = SyntaxParser.FloatToStr(leftItem.getCommonData().cdReal, (int)rightItem.getCommonData().cdReal, threeItem.getCommonData().cdBool);
                        }
                        else if (leftItem.getCommonData().type == 6) {
                            leftItem.getCommonData().cdString = ((leftItem.getCommonData().cdDate == null) ? "" : SyntaxParser.datetimeToStr(leftItem.getCommonData().cdDate));
                        }
                        else {
                            leftItem.getCommonData().cdString = leftItem.getCommonData().toString();
                        }
                        leftItem.getCommonData().type = 3;
                        leftItem.getCommonData().initNullValue();
                        this.disposeNiporlanItem(rightItem);
                        this.disposeNiporlanItem(threeItem);
                        break;
                    }
                    if (item.getFuncName().equals("IDC")) {
                        final NiporlanItem leftItem = this.stack.peek();
                        leftItem.getCommonData().cdString = String.valueOf(SyntaxParser.idc(leftItem.getCommonData().cdString));
                        leftItem.getCommonData().isNull = false;
                        leftItem.getCommonData().initNullValue();
                        break;
                    }
                    if (item.getFuncName().equals("ISDIGIT")) {
                        final NiporlanItem leftItem = this.stack.peek();
                        leftItem.getCommonData().cdBool = SyntaxParser.isDigit(leftItem.getCommonData().cdString);
                        leftItem.getCommonData().type = 5;
                        leftItem.getCommonData().isNull = false;
                        leftItem.getCommonData().initNullValue();
                        break;
                    }
                    if (item.getFuncName().equals("LEN")) {
                        final NiporlanItem leftItem = this.stack.peek();
                        leftItem.getCommonData().cdReal = SyntaxParser.asciiLength(leftItem.getCommonData().cdString);
                        leftItem.getCommonData().type = 2;
                        leftItem.getCommonData().isNull = false;
                        leftItem.getCommonData().initNullValue();
                        break;
                    }
                    if (item.getFuncName().equals("ISNULL")) {
                        final NiporlanItem leftItem = this.stack.peek();
                        leftItem.getCommonData().cdBool = leftItem.getCommonData().isNull;
                        leftItem.getCommonData().type = 5;
                        leftItem.getCommonData().isNull = false;
                        leftItem.getCommonData().initNullValue();
                        break;
                    }
                    if (item.getFuncName().equals("EXISTCHINESE")) {
                        final NiporlanItem leftItem = this.stack.peek();
                        leftItem.getCommonData().cdBool = SyntaxParser.existChinese(leftItem.getCommonData().cdString);
                        leftItem.getCommonData().type = 5;
                        leftItem.getCommonData().isNull = false;
                        leftItem.getCommonData().initNullValue();
                        break;
                    }
                    if (item.getFuncName().equals("INLIST")) {
                        final NiporlanItem leftItem = this.stack.pop();
                        final int count = (int)leftItem.getCommonData().cdReal;
                        leftItem.getCommonData().type = 5;
                        leftItem.getCommonData().cdBool = false;
                        final String[] list = new String[count];
                        for (int l = 0; l < count; ++l) {
                            final NiporlanItem rightItem = this.stack.pop();
                            list[l] = rightItem.getCommonData().cdString;
                            this.disposeNiporlanItem(rightItem);
                        }
                        final NiporlanItem threeItem = this.stack.pop();
                        for (int l = 0; l < count; ++l) {
                            if (list[l].equals(threeItem.getCommonData().cdString) || (!list[l].equals("") && threeItem.getCommonData().cdString.indexOf(list[l]) == 0)) {
                                leftItem.getCommonData().cdBool = true;
                                break;
                            }
                        }
                        this.disposeNiporlanItem(threeItem);
                        leftItem.getCommonData().isNull = false;
                        leftItem.getCommonData().initNullValue();
                        this.stack.push(leftItem);
                        break;
                    }
                    if (item.getFuncName().equals("INDICT")) {
                        final NiporlanItem leftItem = this.stack.pop();
                        final NiporlanItem rightItem = this.stack.peek();
                        final DictMeaningEvent event = new DictMeaningEvent(this);
                        event.code = rightItem.getCommonData().cdString;
                        event.dictName = leftItem.getCommonData().cdString;
                        this.getDictMeaning(event);
                        rightItem.getCommonData().type = 5;
                        rightItem.getCommonData().cdBool = event.inDict;
                        rightItem.getCommonData().isNull = false;
                        rightItem.getCommonData().initNullValue();
                        this.disposeNiporlanItem(leftItem);
                        break;
                    }
                    if (item.getFuncName().equals("DICTVALUE")) {
                        final NiporlanItem threeItem = this.stack.pop();
                        final NiporlanItem leftItem = this.stack.pop();
                        final NiporlanItem rightItem = this.stack.peek();
                        final DictMeaningEvent event = new DictMeaningEvent(this);
                        event.code = rightItem.getCommonData().cdString;
                        event.dictName = leftItem.getCommonData().cdString;
                        event.meaningName = threeItem.getCommonData().cdString;
                        this.getDictMeaning(event);
                        rightItem.getCommonData().assign(event.meaningValue);
                        if (rightItem.getCommonData().type == 1) {
                            rightItem.getCommonData().type = 2;
                            rightItem.getCommonData().cdReal = rightItem.getCommonData().cdInt;
                        }
                        rightItem.getCommonData().initNullValue();
                        this.disposeNiporlanItem(leftItem);
                        this.disposeNiporlanItem(threeItem);
                        break;
                    }
                    if (item.getFuncName().equals("UNITSTATE")) {
                        final NiporlanItem rightItem = this.stack.pop();
                        final NiporlanItem leftItem = this.stack.peek();
                        final UnitStateEvent event2 = new UnitStateEvent(this, leftItem.getCommonData().cdString, rightItem.getCommonData().cdString);
                        this.getUnitState(event2);
                        leftItem.getCommonData().isNull = false;
                        leftItem.getCommonData().type = 2;
                        leftItem.getCommonData().cdReal = event2.state;
                        this.disposeNiporlanItem(rightItem);
                        break;
                    }
                    if (item.getFuncName().equals("LINKPLUS")) {
                        final NiporlanItem threeItem = this.stack.pop();
                        final NiporlanItem leftItem = this.stack.pop();
                        final NiporlanItem rightItem = this.stack.peek();
                        final LinkPlusEvent event3 = new LinkPlusEvent(this, rightItem.getDataCell(), (int)leftItem.getCommonData().cdReal, (int)threeItem.getCommonData().cdReal, this.cellValue);
                        this.getLinkPlus(event3);
                        rightItem.getCommonData().assign(event3.value);
                        if (rightItem.getCommonData().type == 1) {
                            rightItem.getCommonData().type = 2;
                            rightItem.getCommonData().cdReal = rightItem.getCommonData().cdInt;
                        }
                        rightItem.getCommonData().initNullValue();
                        this.disposeNiporlanItem(leftItem);
                        this.disposeNiporlanItem(threeItem);
                        break;
                    }
                    if (item.getFuncName().equals("FLOATSTAT")) {
                        final NiporlanItem leftItem = this.stack.peek();
                        final FloatStatEvent event4 = (FloatStatEvent)leftItem.getObject();
                        this.getFloatStat(event4);
                        leftItem.setType(2);
                        leftItem.getCommonData().assign(event4.value);
                        if (leftItem.getCommonData().type == 1) {
                            leftItem.getCommonData().type = 2;
                            leftItem.getCommonData().cdReal = leftItem.getCommonData().cdInt;
                        }
                        leftItem.getCommonData().initNullValue();
                        leftItem.setObject(null);
                        break;
                    }
                    if (item.getFuncName().equals("FLOATCOPY")) {
                        final NiporlanItem leftItem = this.stack.peek();
                        final FloatCopyEvent event5 = (FloatCopyEvent)leftItem.getObject();
                        this.execFloatCopy(event5);
                        leftItem.setType(2);
                        leftItem.getCommonData().isNull = false;
                        leftItem.getCommonData().type = 2;
                        leftItem.getCommonData().cdReal = event5.copyRowCount;
                        leftItem.setObject(null);
                        break;
                    }
                    if (item.getFuncName().equals("DATEVALUE")) {
                        final NiporlanItem rightItem = this.stack.pop();
                        final NiporlanItem leftItem = this.stack.peek();
                        leftItem.getCommonData().isNull = !SyntaxParser.safeStrToDate(leftItem.getCommonData().getValidcdDate(), leftItem.getCommonData().cdString, rightItem.getCommonData().cdString);
                        leftItem.getCommonData().type = 6;
                        leftItem.getCommonData().initNullValue();
                        break;
                    }
                    if (item.getFuncName().equals("FORMATDATE")) {
                        final NiporlanItem rightItem = this.stack.pop();
                        final NiporlanItem leftItem = this.stack.peek();
                        leftItem.getCommonData().cdString = SyntaxParser.formatDate(leftItem.getCommonData().getValidcdDate(), rightItem.getCommonData().cdString);
                        leftItem.getCommonData().type = 3;
                        leftItem.getCommonData().initNullValue();
                        break;
                    }
                    if (item.getFuncName().equals("DATE")) {
                        final NiporlanItem threeItem = this.stack.pop();
                        final NiporlanItem rightItem = this.stack.pop();
                        final NiporlanItem leftItem = this.stack.peek();
                        SyntaxParser.safeEncodeDate(leftItem.getCommonData().getValidcdDate(), (int)leftItem.getCommonData().cdReal, (int)rightItem.getCommonData().cdReal, (int)threeItem.getCommonData().cdReal);
                        leftItem.getCommonData().type = 6;
                        leftItem.getCommonData().isNull = (leftItem.getCommonData().isNull || rightItem.getCommonData().isNull || threeItem.getCommonData().isNull);
                        leftItem.getCommonData().initNullValue();
                        this.disposeNiporlanItem(rightItem);
                        this.disposeNiporlanItem(threeItem);
                        break;
                    }
                    if (item.getFuncName().equals("YEAR")) {
                        final NiporlanItem leftItem = this.stack.peek();
                        leftItem.getCommonData().cdReal = leftItem.getCommonData().getValidcdDate().get(1);
                        leftItem.getCommonData().type = 2;
                        leftItem.getCommonData().initNullValue();
                        break;
                    }
                    if (item.getFuncName().equals("MONTH")) {
                        final NiporlanItem leftItem = this.stack.peek();
                        leftItem.getCommonData().cdReal = leftItem.getCommonData().getValidcdDate().get(2) + 1;
                        leftItem.getCommonData().type = 2;
                        leftItem.getCommonData().initNullValue();
                        break;
                    }
                    if (item.getFuncName().equals("DAY")) {
                        final NiporlanItem leftItem = this.stack.peek();
                        leftItem.getCommonData().cdReal = leftItem.getCommonData().getValidcdDate().get(5);
                        leftItem.getCommonData().type = 2;
                        leftItem.getCommonData().initNullValue();
                        break;
                    }
                    if (item.getFuncName().equals("TODAY")) {
                        final NiporlanItem leftItem = this.newNiporlanItem();
                        leftItem.setValue(item.getCommonData());
                        leftItem.getCommonData().type = 6;
                        leftItem.getCommonData().cdDate = new GregorianCalendar();
                        leftItem.getCommonData().cdDate.getTime().setTime(0L);
                        this.stack.push(leftItem);
                        break;
                    }
                    final FunctionListener funcListener = this.syntaxParser.getFunctionListener(item.getFuncName());
                    if (funcListener == null) {
                        this.error("没有找到函数“" + item.getFuncName() + "”的侦听器！");
                        return;
                    }
                    final Function func = funcListener.funcByName(item.getFuncName());
                    final CommonDatas paramValues = new CommonDatas();
                    NiporlanItem[] niporlanItems;
                    if (!func.isInfiniteParam) {
                        niporlanItems = new NiporlanItem[func.params.size()];
                        for (int j = 0; j < func.params.size(); ++j) {
                            niporlanItems[func.params.size() - j - 1] = this.stack.pop();
                        }
                    }
                    else {
                        final NiporlanItem leftItem = this.stack.pop();
                        niporlanItems = new NiporlanItem[leftItem.getCommonData().cdInt];
                        for (int j = 0; j < leftItem.getCommonData().cdInt; ++j) {
                            niporlanItems[leftItem.getCommonData().cdInt - j - 1] = this.stack.pop();
                        }
                        this.disposeNiporlanItem(leftItem);
                    }
                    for (int j = 0; j < niporlanItems.length; ++j) {
                        paramValues.add(niporlanItems[j].getCommonData());
                    }
                    final NiporlanItem leftItem = this.newNiporlanItem();
                    leftItem.setType(2);
                    leftItem.getCommonData().type = func.resultType;
                    leftItem.getCommonData().isNull = false;
                    leftItem.getCommonData().initValue();
                    if (funcListener.callFunction(func.name, paramValues, leftItem.getCommonData(), false) != 0) {
                        leftItem.setValue(leftItem.getCommonData());
                    }
                    if (leftItem.getCommonData().type == 1) {
                        leftItem.getCommonData().type = 2;
                        leftItem.getCommonData().cdReal = leftItem.getCommonData().cdInt;
                    }
                    leftItem.getCommonData().initNullValue();
                    this.stack.push(leftItem);
                    break;
                }
                case 2: {
                    final NiporlanItem leftItem = this.newNiporlanItem();
                    leftItem.setValue(item.getCommonData());
                    final CommonData leftItemCommonData = leftItem.getCommonData();
                    if (leftItemCommonData.type == 1) {
                        leftItemCommonData.type = 2;
                        leftItemCommonData.cdReal = leftItemCommonData.cdInt;
                    }
                    this.stack.push(leftItem);
                    break;
                }
                case 3: {
                    final NiporlanItem leftItem = this.newNiporlanItem();
                    leftItem.setValue(item.getDataCell());
                    if (this.useDataLocator) {
                        leftItem.setDataLocator(item);
                    }
                    if (isAssignValue && this.stack.size() == 0) {
                        assignKind = 0;
                    }
                    else {
                        this.syntaxEvent.dataCell = leftItem.getDataCell();
                        this.syntaxEvent.value = this.cellValue;
                        if (this.useDataLocator) {
                            this.getCellValue(this.syntaxEvent, leftItem.getDataLocator());
                        }
                        else {
                            this.getCellValue(this.syntaxEvent);
                        }
                        leftItem.setType(2);
                        final CommonData leftItemCommonData = leftItem.getCommonData();
                        leftItemCommonData.assign(this.syntaxEvent.value);
                        if (leftItemCommonData.type == 1) {
                            leftItemCommonData.type = 2;
                            leftItemCommonData.cdReal = leftItemCommonData.cdInt;
                        }
                        if (leftItemCommonData.type == 0) {
                            this.error("Niporlan公式有错误(空数据类型)！");
                        }
                        leftItemCommonData.initNullValue();
                    }
                    this.stack.push(leftItem);
                    break;
                }
                case 4: {
                    final NiporlanItem leftItem = this.newNiporlanItem();
                    leftItem.setValue(item.getObject());
                    this.stack.push(leftItem);
                    break;
                }
                default: {
                    this.error();
                    break;
                }
            }
        }
    }
    
    private void getCellValue(final SyntaxEvent syntaxEvent) {
        if (this.syntaxParser.getSyntaxListener() != null) {
            this.syntaxParser.getSyntaxListener().getNiValue(syntaxEvent);
        }
    }
    
    private void getCellValue(final SyntaxEvent syntaxEvent, final Object dataLocator) {
        if (this.syntaxParser.getOptimizeListener() != null) {
            this.syntaxParser.getOptimizeListener().getCellValue(syntaxEvent, dataLocator);
        }
    }
    
    private void setCellValue(final SyntaxEvent syntaxEvent) {
        if (this.syntaxParser.getSyntaxListener() != null) {
            this.syntaxParser.getSyntaxListener().setNiValue(syntaxEvent);
        }
    }
    
    private void setCellValue(final SyntaxEvent syntaxEvent, final Object dataLocator) {
        if (this.syntaxParser.getOptimizeListener() != null) {
            this.syntaxParser.getOptimizeListener().setCellValue(syntaxEvent, dataLocator);
        }
    }
    
    private void getStatValue(final SyntaxEvent syntaxEvent) {
        if (this.syntaxParser.getSyntaxListener() != null) {
            this.syntaxParser.getSyntaxListener().getStatValue(syntaxEvent);
        }
    }
    
    private void getUnitState(final UnitStateEvent unitStateEvent) {
        if (this.syntaxParser.getSyntaxListener() != null) {
            this.syntaxParser.getSyntaxListener().getUnitState(unitStateEvent);
        }
    }
    
    private void getLinkPlus(final LinkPlusEvent linkPlusEvent) {
        if (this.syntaxParser.getSyntaxListener() != null) {
            this.syntaxParser.getSyntaxListener().getLinkPlus(linkPlusEvent);
        }
    }
    
    private void getFloatStat(final FloatStatEvent floatStatEvent) {
        if (this.syntaxParser.getSyntaxListener() != null) {
            this.syntaxParser.getSyntaxListener().getFloatStat(floatStatEvent);
        }
    }
    
    private void execFloatCopy(final FloatCopyEvent floatCopyEvent) {
        if (this.syntaxParser.getSyntaxListener() != null) {
            this.syntaxParser.getSyntaxListener().execFloatCopy(floatCopyEvent);
        }
    }
    
    private void getDictMeaning(final DictMeaningEvent dictMeaningEvent) {
        if (this.syntaxParser.getSyntaxListener() != null) {
            this.syntaxParser.getSyntaxListener().getDictMeaning(dictMeaningEvent);
        }
    }
    
    private void error() {
        this.error("Niporlan公式有错误 或 不支持的操作！");
    }
    
    private void error(final String message) {
        System.out.println(message);
    }
}
